/**
	 *The Program may contain sample source code or programs, which illustrate programming techniques. These samples are found in the Samples Gallery.  You may copy, modify, and distribute these samples, or their modifications, in any form, internally or as part of your application or related documentation. These samples have not been tested under all conditions and are provided to you by IBM without obligation of support of any kind.  IBM PROVIDES THESE SAMPLES  "AS IS", SUBJECT TO ANY STATUTORY WARRANTIES THAT CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
	 */

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.ecore.EObject;

import com.ibm.xtools.pluglets.Pluglet;
import com.ibm.xtools.emf.msl.ResourceSetReadOperation;
import com.ibm.xtools.modeler.UMLModeler;

import com.ibm.xtools.notation.View;

public class ModelEnumeration extends Pluglet {

	/**
	 * Walk the selected objects and log them to the console
	 */
	public void plugletmain(String[] args) {

		try {
			UMLModeler.getEditingDomain().run( new ResourceSetReadOperation() {

				protected void execute(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {

					// Get selection
					final List elements = UMLModeler.getUMLUIHelper().getSelectedElements();
					// Keep track if we log anything
					boolean logged = false;
					
					// Log each elements
					for (Iterator iter = elements.iterator(); iter.hasNext();) {
						Object object = iter.next();
						
						// If a view, try to get semantic bridge
						if (object instanceof View) {
							object = ((View)object).getElement();					}

						// If a class
						if (object instanceof org.eclipse.uml2.Class) {
							org.eclipse.uml2.Class clazz = (org.eclipse.uml2.Class) object;
							logObject(clazz, ""); //$NON-NLS-1$
							logged = true;
						}
					}

					if (false == logged) {
						out.println("Could not enumerate selection"); //$NON-NLS-1$
					}

				}
			},new NullProgressMonitor());
			
		} catch (InvocationTargetException e) {
			out.println("An unexpected exception prevented the operation from completing successfully. See Error Log View for details."); //$NON-NLS-1$
			throw new RuntimeException(e.getCause());
		} catch (InterruptedException e) {
			out.println("The operation was interrupted"); //$NON-NLS-1$
		}

	}


	/**
	 * Logs the specified object's toString() and its content
	 * @param object The object to log 
	 * @param indent The string to prefix to the object's toString()
	 */
	private void logObject(Object object, String indent) {

		// Log object
		if (null != object) {
			out.println(indent + object.toString());
		} else {
			out.println(indent + "<null>"); //$NON-NLS-1$
		}
		
		// Log contents
		if (object instanceof EObject) {
			Iterator contents = ((EObject)object).eContents().iterator();
			while (contents.hasNext()) {
				logObject(contents.next(), indent + "\t"); //$NON-NLS-1$
			}
		}
	}

}
